addpath forward

% Load structure containing electrode labels, 
% electrode positions, and everything needed to perform forward
% calculations in the icbm 152 average head provided by MNI.
% Coordinates in naspalparini, vc{:}.vc, head.vc, cortex.vc, locs_3D, 
% refloc, and grid_coarse are in MNI space (unit: cm) after flipping the sign 
% of the first two coordinates. 
% Note that in the original Neuroimage publicatoin the Colins 27 head model 
% was used instead of the average head model.
par = load('icbm152b_sym'); 

par.Nexp = 100; % number of random datasets to be generated
par.N = 10000; % length of time series
par.M = length(par.sa.clab_electrodes); % number of EEG electrodes
par.Kgen = 5; % order of signal AR model
par.gamma = 0.5; % (signal-to-noise ratio = 1)

% place two dipoles with tangential orientation 3cm under C3 and C4
[is ia ib] = intersect(par.sa.clab_electrodes, {'C3', 'C4'}); 
[x1 y1] = pol2cart(pi, 1);
[x2 y2] = pol2cart(pi, 1);
par.dips = [par.sa.locs_3D(ia, 1:3) - 3*par.sa.locs_3D(ia, 4:6) [[x1 y1]*null(par.sa.locs_3D(ia(1), 4:6))'; [x2 y2]*null(par.sa.locs_3D(ia(2), 4:6))']];

% generate corresponding EEG field pattern using precomputed lead field
par.pat = [];
[mi ind_dip1] = min(sqrt(sum((par.sa.grid_coarse-repmat(par.dips(1, 1:3), 2113, 1)).^2, 2)));
[mi ind_dip2] = min(sqrt(sum((par.sa.grid_coarse-repmat(par.dips(2, 1:3), 2113, 1)).^2, 2)));
par.pat(:, 1) = squeeze(par.sa.V_coarse(:, ind_dip1, :))*par.dips(1, 4:6)';
par.pat(:, 2) = squeeze(par.sa.V_coarse(:, ind_dip2, :))*par.dips(2, 4:6)';

% name of simulation series
par.simulation = 'neuroimage';

mkdir(['simulations/' par.simulation]);
save(['simulations/' par.simulation '/par'], 'par');


% these parts are sampled randomly in each experiment  
RandStream.setDefaultStream(RandStream('mt19937ar','seed', sum(100*clock)));
for iexp = 1:par.Nexp
  data = [];
  
  % generate bivariate source time series with unidirectional flow
  [data.z data.A data.inn] = gen_ar_biv(par.N, par.Kgen);

  % signal projection to sensors
  data.signal = par.pat*data.z;
  data.signal = data.signal ./ norm(data.signal, 'fro');  
  
  % noise
  data.noise = [];

  % biological noise
  data.noiseinds = ceil(rand(10, 1)*size(par.sa.grid_coarse, 1));
  for ibr = 1:length(data.noiseinds)
    data.noise(ibr, :) = gen_ar_uni(par.N, 10);
  end
  data.noise = reshape(par.sa.V_coarse(:, data.noiseinds, :), par.M, [])*(repmat(randn(30, 1), 1, par.N).*reshape(repmat(reshape(data.noise, 1, 10, par.N), [3, 1, 1]), 30, par.N));
  data.noise = data.noise ./ norm(data.noise, 'fro');

  % plus sensor noise
  sensor_noise = randn(size(data.noise));
  sensor_noise = sensor_noise ./ norm(sensor_noise, 'fro');
  data.noise = data.noise + sensor_noise;
  data.noise = data.noise ./ norm(data.noise, 'fro');
  
  % EEG sensor measurement = signal + noise
  data.eeg = par.gamma*data.signal + (1-par.gamma)*data.noise;

  save(['simulations/' par.simulation '/exp' num2str(iexp)], 'data');
end

